package computecolumn;
import java.awt.BorderLayout;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.ButtonModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.border.EtchedBorder;

/**
 * A dialog box to select the columns and functions to compute a new column in
 * the table.
 * 
 * @author Etienne Fort
 */
public class ComputeColumnDialog extends JDialog implements ActionListener {
	private class RadioGroupPanel extends JPanel {
		private ButtonGroup itemsGroup;
		public RadioGroupPanel(String[] items, int axis) {
			setLayout(new BoxLayout(this, axis));
			itemsGroup = new ButtonGroup();
			for (int i = 0; i < items.length; i++) {
				JRadioButton r = new JRadioButton(items[i], i == 0);
				itemsGroup.add(r);
				r.setActionCommand(r.getText());
				add(r);
			}
		}
		public ButtonModel getSelection() {
			return itemsGroup.getSelection();
		}
	}
	public static final String stCancel = "Annuler";
	public static final String[] stFunctions = {"1/2", "exp", "log", "1/x"};
	public static final String stOk = "Ok";
	/**
	 * Get the name of the clicked button. Comment for
	 * <code>buttonClicked</code>
	 */
	private String buttonClicked = "";
	private JComboBox column1Box;
	private JComboBox column2Box;
	private RadioGroupPanel functionBox;
	/**
	 * Creates a new dialog frame.
	 * 
	 * @param frame
	 *            the frame from which the dialog box is displayed.
	 * @param title
	 *            the title of the dialog box.
	 * @param columnNames
	 *            the names of the columns to be displayed in combo boxes.
	 */
	public ComputeColumnDialog(Frame frame, String title, String[] columnNames) {
		super(frame, title, true);
		JPanel panel = new JPanel();
		Box panelCombos = Box.createVerticalBox();
		panelCombos.add(column1Box = new JComboBox(columnNames));
		panelCombos.add(column2Box = new JComboBox(columnNames));
		JPanel panelFunction = new JPanel();
		panelFunction.setBorder(BorderFactory.createTitledBorder(BorderFactory
				.createEtchedBorder(EtchedBorder.RAISED), "Fonction : "));
		panelFunction.add(functionBox = new RadioGroupPanel(stFunctions,
				BoxLayout.X_AXIS));
		panel.add(panelCombos);
		panel.add(panelFunction);
		JPanel panelBottom = new JPanel();
		JButton buttonCancel = new JButton(stCancel);
		panelBottom.add(buttonCancel);
		buttonCancel.addActionListener(this);
		JButton buttonOk = new JButton(stOk);
		panelBottom.add(buttonOk);
		buttonOk.addActionListener(this);
		getContentPane().add(panel);
		getContentPane().add(panelBottom, BorderLayout.SOUTH);
		getRootPane().setDefaultButton(buttonOk);
	}
	/**
	 * Listens to buttons related events.
	 * 
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		String command = e.getActionCommand();
		if ((command.equals(stCancel)) || (command.equals(stOk))) {
			buttonClicked = command;
			dispose();
		}
	}
	/**
	 * @return the clicked button
	 */
	public String getButtonClicked() {
		return buttonClicked;
	}
	/**
	 * @return the first column to calculate with.
	 */
	public int getColumn1() {
		return column1Box.getSelectedIndex();
	}
	/**
	 * @return the second column to calculate with.
	 */
	public int getColumn2() {
		return column2Box.getSelectedIndex();
	}
	/**
	 * @return the function to calculate.
	 */
	public String getFunction() {
		return functionBox.getSelection().getActionCommand();
	}
}