package computecolumn;
import javax.swing.table.TableModel;

import tableoperations.TableMap;
/**
 * A Table to calculate a new column given a function and two column indexes. It
 * Extends TableMap so it can be saved in a file.
 * 
 * @author Julien Robert
 */
public class TableAddFunctionColumn extends TableMap {
	/**
	 * Computes the given function on both operands a and b.
	 * @param function 
	 * 				the string which specifies the function to use
	 * @param a 
	 * 				the first operand
	 * @param b 
	 * 				the second operand
	 * @return the result of the function calculated with both operands
	 */	
	public static double calculateWithFunction(String function, double a, double b) {
		if (function.equals(ComputeColumnDialog.stFunctions[0]))
			return (a / b);
		if (function.equals(ComputeColumnDialog.stFunctions[1]))
			return Math.log(a);
		if (function.equals(ComputeColumnDialog.stFunctions[2]))
			return Math.exp(a);
		if (function.equals(ComputeColumnDialog.stFunctions[3]))
			return (1 / a);
		return (0);
	}
	private int fromColumn1, fromColumn2;
	private String fromFunction;
	/**
	 * Creates a new <code>TableAddFunctionColumn</code> given a model and two columns to compute with the function.
	 * @param model 
	 * 				the model <code>TableModel</code>
	 * @param column1 
	 * 				the index of the first column to compute from.
	 * @param column2 
	 * 				the index of the second column to compute from.
	 * @param function 
	 * 				the string specifying the function to use.
	 */
	public TableAddFunctionColumn(TableModel model, int column1, int column2,
			String function) {
		setModel(model);
		fromColumn1 = column1;
		fromColumn2 = column2;
		fromFunction = function;
	}
	/** 
	 * @see javax.swing.table.TableModel#getColumnClass(int)
	 */
	public Class getColumnClass(int aColumn) {
		int diff = aColumn - model.getColumnCount();
		if (diff < 0) // We are in the original Table
			return super.getColumnClass(aColumn);
		else
			return (Double.class);
	}
	/**
	 * @see javax.swing.table.TableModel#getColumnCount()
	 */
	public int getColumnCount() {
		return super.getColumnCount() + 1;
	}
	/**
	 * @see javax.swing.table.TableModel#getColumnName(int)
	 */
	public String getColumnName(int aColumn) {
		int diff = aColumn - super.getColumnCount();
		if (diff < 0) // We are in the original Table
			return super.getColumnName(aColumn);
		else
			if (fromFunction.equals(ComputeColumnDialog.stFunctions[0]))
			return super.getColumnName(fromColumn1) + fromFunction
					+ super.getColumnName(fromColumn2);
			else if (fromFunction.equals(ComputeColumnDialog.stFunctions[1])) return "exp("+super.getColumnName(fromColumn1)+")";
			else if (fromFunction.equals(ComputeColumnDialog.stFunctions[2])) return "log("+super.getColumnName(fromColumn1)+")";
			else if (fromFunction.equals(ComputeColumnDialog.stFunctions[3])) return "1/("+super.getColumnName(fromColumn1)+")";
			else return "";
	}
	/** 
	 * @see javax.swing.table.TableModel#getValueAt(int, int)
	 */
	public Object getValueAt(int aRow, int aColumn) {
		int diff = aColumn - super.getColumnCount();
		if (diff < 0) // We are in the original Table
			return super.getValueAt(aRow, aColumn);
		else {        //We are not, so return the computed column
			double result, d1, d2 = Double.NaN;
			Object cell = model.getValueAt(aRow, fromColumn1);
			if (cell instanceof Double)
				d1 = ((Double) cell).doubleValue();
			else
				return (new Double(Double.NaN));
			cell = model.getValueAt(aRow, fromColumn2);
			if (fromFunction.equals(ComputeColumnDialog.stFunctions[0]))
				if (cell instanceof Double)
					d2 = ((Double) cell).doubleValue();
				else
					return (new Double(Double.NaN));
			
			result = calculateWithFunction(fromFunction, d1, d2);
			return new Double(result);
		}
	}
	/**
	 *  
	 * @see javax.swing.table.TableModel#isCellEditable(int, int)
	 */
	public boolean isCellEditable(int row, int column) {
		int diff = column - super.getColumnCount();
		if (diff < 0)
			return super.isCellEditable(row, column);
		else
			return false;
	}
	/**
	 * @see javax.swing.table.TableModel#setValueAt(java.lang.Object, int, int)
	 */
	public void setValueAt(Object aValue, int aRow, int aColumn) {
		int diff = aColumn - super.getColumnCount();
		if (diff < 0) // We are in the original Table
			super.setValueAt(aValue, aRow, aColumn);
	}
}
