package graphs;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.print.Printable;

import javax.swing.JPanel;
import javax.swing.table.TableModel;
/**
 * Graphic representation of the results of filtering in terms of variable
 * thresholds. Displays one or two curves according as there are one or two
 * columns implied; that is the reason why there is an inheritance.
 * 
 * @see GraphPanel1 and GraphPanel2. 
 * The width of the graph at the ordinate y represents 
 * the number of genes that have been selected by filtering.
 * 
 * @author Thomas Hugel
 */
public abstract class GraphPanel extends JPanel  implements Printable{
	public static final Dimension GRAPH_DIMENSION = new Dimension(300,300);
	protected int fontSize;
	protected int middleAbscissa;
	protected TableModel originalTable;
	protected double scaleH, scaleW;
	protected int verticalResolution;
	protected int widthNb;
	/**
	 * 
	 * @param model
	 *            the original table from which the graphs are drawn
	 * @param vRes
	 *            the vertical resolution  
	 */
	public GraphPanel(TableModel model, int vRes) {
		originalTable = model;
		verticalResolution = vRes;
	}
	/**
	 * Draws the graph by linear interpolation of coords.
	 * 
	 * @param g
	 * 				the graphic area
	 * @param color
	 * 				the color of the graph
	 * @param coords
	 * 				the points that are interpolated 				
	 */
	public void drawCurve(Graphics g, Color color, Point[] coords) {
		g.setColor(color);
		for (int i = 0; i < 2 * verticalResolution - 1; i++) {
			if ((coords[i].x == middleAbscissa)
					&& (coords[i + 1].x == middleAbscissa))
				g.setColor(Color.BLACK);
			else
				g.setColor(color);
			if (i != verticalResolution - 1)
				g.drawLine(coords[i].x, coords[i].y, coords[i + 1].x,
						coords[i + 1].y);
		}
	}
	/**
	 *  
	 * @param y
	 *            the current ordinate
	 * @param thrRange
	 *            the range of thresholds (min and max)
	 * @return the threshold corresponding to the ordinate y
	 */
	public double ordToThreshold(int y, double[] thrRange) {
		return (thrRange[1] + ((double) y / (double) verticalResolution)
				* (thrRange[0] - thrRange[1]));
	}
	/**
	 * Sets the graphic parameters
	 * 
	 * @see java.awt.Component#paint(java.awt.Graphics)
	 */
	public void paint(Graphics g) {
		super.paint(g);
		setBackground(Color.WHITE);
		setPreferredSize(GRAPH_DIMENSION);
		middleAbscissa = new Double(getWidth() / 2.).intValue();
		fontSize = (int) (g.getFont().getSize2D()) + 1;
		scaleH = getHeight() / (double) verticalResolution;
		scaleW = getWidth() / (double) widthNb;
		g.drawString("nb gnes", middleAbscissa - 10, (int) getHeight() - 3);
	}
}