package graphs;
import filters.FilterParam;
import filters.TableParamsFilter;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.print.PageFormat;
import java.awt.print.PrinterException;

import javax.swing.table.TableModel;
/**
 * Subclass of GraphPanel for a unique graph.
 * 
 * @see graphs.GraphPanel
 * @author Thomas Hugel
 */
public class GraphPanel1 extends GraphPanel {
	private Color COLOR;
	private Point[] coords;
	protected Point[] filteringResults;
	/**
	 * Constructs the unique graph dynamically in terms of input thresholds.
	 * 
	 * @param model
	 *            	the support of the filtering.
	 * @param param
	 *            	the input parameters of the filtering.
	 * @param vRes	 
	 * 				the vertical resolution
	 * @param range 
	 * 				the range of thresholds (min and max)
	 */
	public GraphPanel1(TableModel model, FilterParam param, int vRes,
			double[] range, Color color) {
		super(model, vRes);
		COLOR = color;
		coords = new Point[2 * verticalResolution];
		FilterParam[] paramsArray = new FilterParam[1];
		paramsArray[0] = param;
		TableParamsFilter filteredTable = new TableParamsFilter(model,
				paramsArray);
		filteringResults = calculateFilteringResults(param, filteredTable,
				range);
		widthNb = filteringResults[vRes - 1].x;
	}
	/**
	 * Commands the starting of the filtering and tabulates the results.
	 * 
	 * @param param
	 *            	the input parameters for the filtering
	 * @param filteredTable
	 *            	the filtered table
	 * @param thrRange 
	 * 				the range of thresholds to explore (min and max)
	 * @return the results of the filtering for the different threshold values.
	 */
	public Point[] calculateFilteringResults(FilterParam param,
			TableParamsFilter filteredTable, double[] thrRange) {
		Point[] results = new Point[verticalResolution];
		boolean OLD_VAL = FilterParam.ALERT_LISTENERS;
		FilterParam.ALERT_LISTENERS = false;
		for (int i = 0; i < verticalResolution; i++) {
			param.setThreshold(ordToThreshold(i, thrRange));
			filteredTable.reallocateIndexes();
			results[i] = new Point(filteredTable.getRowCount(), i);
		}
		FilterParam.ALERT_LISTENERS = OLD_VAL;
		return (results);
	}
	/**
	 * @param singleResults
	 *            results of a single filtering
	 * @return the coordinates of the graph when it is unique (vertical symmetry)
	 */
	public Point[] computeCoordsMiddle(Point[] singleResults) {
		for (int i = 0; i < verticalResolution; i++) {
			coords[verticalResolution - i - 1] = new Point(
					(int) (middleAbscissa - (singleResults[i].x) * scaleW / 2.),
					(int) ((singleResults[i].y) * scaleH));
			coords[i + verticalResolution] = new Point(
					(int) (middleAbscissa + (singleResults[i].x) * scaleW / 2.),
					coords[verticalResolution - i - 1].y);
		}
		return (coords);
	}
	/**
	 * @see java.awt.Component#paint(java.awt.Graphics)
	 */
	public void paint(Graphics graph) {
		super.paint(graph);
		Point[] coords = computeCoordsMiddle(filteringResults);
		drawCurve(graph, COLOR, coords);
		graph.drawString("seuil", 1, fontSize);
	}
	
	/**
	 * @see java.awt.print.Printable#print(java.awt.Graphics, java.awt.print.PageFormat, int)
	 */
	public int print(Graphics g, PageFormat pageFormat, int nbPages) throws PrinterException {
		if (nbPages>=1) return NO_SUCH_PAGE;
		else 
			{
			double X= pageFormat.getImageableX();
			double Y= pageFormat.getImageableY();
			double width = pageFormat.getImageableWidth();
			double height = pageFormat.getImageableHeight();
			
			middleAbscissa = new Double(width/2).intValue() + new Double(X).intValue();
			int fontSize = (int) (g.getFont().getSize2D()) + 1;
			scaleH = height / (double) verticalResolution;
			scaleW = width / (double) widthNb;
			g.drawString("nb gnes", middleAbscissa - 10, (int) height - 3);
			Point[] coords = computeCoordsMiddle(filteringResults);
			drawCurve(g, COLOR, coords);
			g.drawString("seuil", 1+(int)X, fontSize + (int)Y);
			

			return PAGE_EXISTS;
	
			}
	}
}