package graphs;
import java.awt.GridLayout;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.Box;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.table.TableModel;

import filters.FilterParam;
/**
 * This class defines the panel that will contain the parameters of the graphs
 * 
 * @author Julien Robert, Etienne Fort
 */
public class GraphsParamsPanel extends JPanel {
	protected class GraphsParamsLine extends JPanel {
		JComboBox columnBox;
		private JLabel labelMax;
		private JLabel labelMin;
		JSlider maxSlider;
		JSlider minSlider;
		private FilterParam parameter;
		
		/**
		 * Place where the parameters of one of the graphs are defined.
		 * 
		 * @param title
		 * 				name of the graph
		 * @param columnNames
		 * 				names of the column of the combo box
		 * @param num
		 * 				number of the GraphsParamsLine
		 */
		public GraphsParamsLine(String title, Object[] columnNames, final int num) {
			JPanel columnPanel = new JPanel(new GridLayout(0, 1));
			columnPanel.add(checkbox[num] = new JCheckBox(title, true));
			checkbox[num].addChangeListener(new ChangeListener() {
				/* 
				 * @see javax.swing.event.ChangeListener#stateChanged(javax.swing.event.ChangeEvent)
				 */
				public void stateChanged(ChangeEvent e) {
					setEnabled(checkbox[num].isSelected());
					firePropertyChange("OutOfDate", false, true);
				}
			});

			columnPanel.add(columnBox = new JComboBox(columnNames));
			Box graphPanel = Box.createVerticalBox();
			JPanel thresholdPanel = new JPanel();
			thresholdPanel.add(labelMin = new JLabel("Min: "));
			thresholdPanel.add(minSlider = new JSlider(JSlider.HORIZONTAL, 0, 1000, 0));
			JPanel thresholdPanel2 = new JPanel();
			thresholdPanel2.add(labelMax = new JLabel("Max: "));
			thresholdPanel2.add(maxSlider = new JSlider(JSlider.HORIZONTAL, 0, 1000, 1000));
			graphPanel.add(thresholdPanel);
			graphPanel.add(thresholdPanel2);
			add(columnPanel);
			add(graphPanel);
			minSlider.addChangeListener(new ChangeListener() {
				public void stateChanged(ChangeEvent arg0) {
					//To avoid min > max
					if (maxSlider.getValue() < minSlider.getValue())
						maxSlider.setValue(minSlider.getValue());
					firePropertyChange("OutOfDate", false, true);
				}
			});
			maxSlider.addChangeListener(new ChangeListener() {
				public void stateChanged(ChangeEvent arg0) {
					if (maxSlider.getValue() < minSlider.getValue())
						minSlider.setValue(maxSlider.getValue());
					firePropertyChange("OutOfDate", false, true);
				}
			});
			columnBox.addItemListener(new ItemListener() {
				/* (non-Javadoc)
				 * @see java.awt.event.ItemListener#itemStateChanged(java.awt.event.ItemEvent)
				 */
				public void itemStateChanged(ItemEvent arg0) {
					parameter.setColumnIndex(columnBox.getSelectedIndex());
					parameter.setColumnText(columnBox.getSelectedItem()
							.toString());
					firePropertyChange("OutOfDate", false, true);
				}
			});
			parameter = new FilterParam(columnBox.getSelectedIndex(), 
					columnBox.getSelectedItem().toString(), getThresholdRange()[0]);
		}
		/**
		 * @return the parameter corresponding to the line
		 */
		public FilterParam getParameter() {
			return parameter;
		}
		/**
		 * @return the thresholds range (min - max)
		 */
		public double[] getThresholdRange() {
			double[] result = new double[2];
			result[0] = (double) minSlider.getValue()
					/ (double) minSlider.getMaximum();
			result[1] = (double) maxSlider.getValue()
					/ (double) maxSlider.getMaximum();
			return result;
		}
		/* (non-Javadoc)
		 * @see java.awt.Component#setEnabled(boolean)
		 */
		public void setEnabled(boolean enabled) {
			columnBox.setEnabled(enabled);
			maxSlider.setEnabled(enabled);
			minSlider.setEnabled(enabled);
			labelMax.setEnabled(enabled);
			labelMin.setEnabled(enabled);
			super.setEnabled(enabled);
		}
	}
	private JCheckBox[] checkbox = new JCheckBox[2];
	private String[] columnNames;
	private GraphsParamsLine graph1, graph2;
	protected TableModel originalTable;
	/**
	 * 
	 * @param nbLines
	 * 				number of graphs
	 * @param table
	 * 				the support of the calculus
	 */
	public GraphsParamsPanel(int nbLines, TableModel table) {
		this.originalTable = table;
		columnNames = new String[table.getColumnCount()];
		for (int i = 0; i < columnNames.length; i++) {
			columnNames[i] = table.getColumnName(i);
		}
		Box box;
		add(box = Box.createVerticalBox());
		box.add(graph1 = new GraphsParamsLine("Courbe 1 : ", columnNames, 0));
		box.add(graph2 = new GraphsParamsLine("Courbe 2 : ", columnNames, 1));
		PropertyChangeListener propertyChangeListener = new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent evt) {
				firePropertyChange("OutOfDate", false, true);
			}
		};
		graph1.addPropertyChangeListener(propertyChangeListener);
		graph2.addPropertyChangeListener(propertyChangeListener);
	}
	/**
	 * @return the number of graphs (0, 1 or 2)
	 */
	public int getNbCurves() {
		return (checkbox[0].isSelected() ? 1 : 0) + (checkbox[1].isSelected() ? 1 : 0);
	}
	/**
	 * @return the number of the unique graph (0 or 1)
	 */
	public int getNumCurve(){
		return (checkbox[0].isSelected() ? 0 : 1);
	}
	/**
	 * Gets all input data.
	 * 
	 * @return an array with all input data
	 * @see FilterParam
	 */
	public FilterParam[] getParameters() {
		FilterParam[] result = new FilterParam[2];
		result[0] = graph1.getParameter();
		result[1] = graph2.getParameter();
		return result;
	}
	/**
	 * @return the range of thresholds (min - max) of the first graph
	 */
	public double[] getThresholdRange1() {
		return graph1.getThresholdRange();
	}
	/**
	 * @see GraphsParamsPanel#getThresholdRange1()
	 */
	public double[] getThresholdRange2() {
		return graph2.getThresholdRange();
	}

}