package tableoperations;
import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Vector;

import javax.swing.table.DefaultTableModel;
/**
 * A <code>DefaultTableModel</code> which is able to read its contents from a file at the
 * CSV format.
 * 
 * @author Julien Robert
 */
public class TableFile extends DefaultTableModel {
	Vector columnClasses = new Vector();
	/**
	 * Creates a new table giving the name of the file.
	 * 
	 * @param fileName
	 *          	  name of the file to be read.
	 * @throws IOException
	 */
	public TableFile(String fileName) throws IOException {
		readFromFileAsCsv(fileName, true);
	}
	/** 
	 * @see javax.swing.table.TableModel#getColumnClass(int)
	 * 
	 * @return the class of the column designated by columnIndex.
	 */
	public Class getColumnClass(int columnIndex) {
		return (Class) columnClasses.get(columnIndex);
	}
	/**
	 * Builds the table reading the file.
	 * 
	 * @param fileName
	 *            the name of the file.
	 * @param fileHasHeaders
	 *            indicates if columns in the file have headers.
	 * @throws IOException
	 *             in case of a problem during the process.
	 */
	public void readFromFileAsCsv(String fileName, boolean fileHasHeaders)
			throws IOException {
		int row, col;
		boolean inAString;
		String ligne;
		String token;
		row = col = 0; //initialisations
		if (fileName == "")
			return;
		InputStream ips = new FileInputStream(fileName);
		InputStreamReader ipsr = new InputStreamReader(ips);
		BufferedReader br = new BufferedReader(ipsr);
		columnIdentifiers.clear();
		columnClasses.clear();
		ips = new FileInputStream(fileName);
		ipsr = new InputStreamReader(ips);
		br = new BufferedReader(ipsr);
		while ((ligne = br.readLine()) != null) {
			int j = 0;
			col = 0; //on se replace au niveau de la premire colonne
			inAString = false; //on n'est pas dans un mot
			token = "";
			for (int i = 0; i < ligne.length(); i++) { //on parcourt la ligne
				if ((ligne.charAt(i) == ';') && (!inAString))
					token = ligne.substring(j, i);
				else if (ligne.charAt(i) == '"') { // On rentre dans un mot
					if (i != ligne.length() - 1) {
						if (ligne.charAt(i + 1) == '"')
							if (!inAString)
								col++;
					}
					if (inAString) {
						token = ligne.substring(j, i);
						inAString = false;
					} else
						inAString = true;
					j = ++i; // on incrmente i pour ne pas avoir les
					// guillemets, et j devient ce nouveau i pour
					// marquer le dbut du mot
				} else if (i == ligne.length() - 1)
					token = ligne.substring(j, i + 1);
				if (!token.equals("")) {
					if (row == 0)
						columnClasses.add(Double.class);
					Object convertedToken;
					try {
						convertedToken = new Double(token.replace(',', '.'));
					} catch (NumberFormatException e) {
						convertedToken = token;
						if ((!fileHasHeaders) || !(row == 0))
							columnClasses.set(col, Object.class);
					}
					if (fileHasHeaders) {
						setRowCount(row);
						if (row == 0)
							columnIdentifiers.add(convertedToken);
						else
							setValueAt(convertedToken, row - 1, col);
					} else {
						setRowCount(row + 1);
						setValueAt(convertedToken, row, col);
					}
					j = 1 + i; // on met j  i + 1 pour ne pas avoir le
					// point virgule au prochain passage
					col++; // on passe a la case d' cot
					token = "";
				}
			}
			row++;
			setColumnCount(col);
		}
		br.close();
	}
}
