package venndiagrams;
import java.awt.GridLayout;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.Vector;

import javax.swing.Box;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.table.TableModel;

import filters.FilterParam;

/**
 * A panel for the filterParams of the table treatment. Inner classes are defined
 * in order to layout the different graphic components.
 * 
 * @author Etienne Fort, Thomas Hugel
 */
public class FiltersParamsPanel extends JPanel {
	/**
	 * A panel containing the widgets used to input parameters for a filtering
	 * 
	 * @author Etienne Fort
	 */
	protected class FilterLine extends JPanel {
		JComboBox columnBox;
		private FilterParam parameter;
		JSlider thresholdSlider;
		public FilterLine(String title, Object[] columnNames) {
			JPanel columnPanel = new JPanel(new GridLayout(0, 1));
			columnPanel.add(new JLabel(title));
			columnPanel.add(columnBox = new JComboBox(columnNames));
			Box filterPanel = Box.createVerticalBox();
			JPanel thresholdPanel = new JPanel(new GridLayout(0, 1));
			thresholdPanel.add(new JLabel("Seuil : "));
			thresholdPanel.add(thresholdSlider = new JSlider(JSlider.HORIZONTAL, 0, 1000, 200));
			filterPanel.add(thresholdPanel);
			add(columnPanel);
			add(filterPanel);
			thresholdSlider.addChangeListener(new ChangeListener() {
				/* (non-Javadoc)
				 * @see javax.swing.event.ChangeListener#stateChanged(javax.swing.event.ChangeEvent)
				 */
				public void stateChanged(ChangeEvent arg0) {
					parameter.setThreshold(getThresholdValue());
				}
			});
			columnBox.addItemListener(new ItemListener() {
				/* (non-Javadoc)
				 * @see java.awt.event.ItemListener#itemStateChanged(java.awt.event.ItemEvent)
				 */
				public void itemStateChanged(ItemEvent arg0) {
					parameter.setColumnIndex(columnBox.getSelectedIndex());
					parameter.setColumnText(columnBox.getSelectedItem().toString());
				}
			});
			parameter = new FilterParam(columnBox.getSelectedIndex(), 
					columnBox.getSelectedItem().toString(), getThresholdValue());
		}
		public FilterParam getParameter() {
			return parameter;
		}
		/**
		 * @return the threshold as a double value between 0 and 1, according to
		 *         the slider.
		 */
		public double getThresholdValue() {
			return (double) thresholdSlider.getValue()
					/ (double) thresholdSlider.getMaximum();
		}
	}
	private Box box;
	private String[] columnNames;
	protected Vector filterLines;
	private int nbFilterLines = 0;
	protected TableModel originalTable;
	/**
	 * Creates a new FiltersParamsPanel given a model TableModel and the number
	 * of initial parameters.
	 * 
	 * @param nbLines
	 *            the initial number of lines.
	 * @param table
	 *            the model to compute on.
	 */
	public FiltersParamsPanel(int nbLines, TableModel table) {
		this.originalTable = table;
		columnNames = new String[table.getColumnCount()];
		for (int i = 0; i < columnNames.length; i++) {
			columnNames[i] = table.getColumnName(i);
		}
		filterLines = new Vector();
		add(box = Box.createVerticalBox());
		setNbFilterLines(nbLines);
	}
	/**
	 * Gets all input data.
	 * 
	 * @return an array with all input data
	 * @see FilterParam
	 */
	public FilterParam[] getParameters() {
		FilterParam[] result = new FilterParam[filterLines.size()];
		for (int i = 0; i < filterLines.size(); i++)
			result[i] = ((FilterLine) filterLines.get(i)).getParameter();
		return result;
	}
	/**
	 * Defines the number of lines "parameters" to choose inside the panel.
	 * Allocates dynamically the proper number of widgets.
	 * 
	 * @param nbLines
	 *            The nbLines to set.
	 */
	public void setNbFilterLines(int nbLines) {
		for (int i = nbFilterLines - 1; i >= nbLines; i--) {
			box.remove((FilterLine) filterLines.get(i));
		}
		filterLines.setSize(nbLines);
		for (int i = nbFilterLines; i < nbLines; i++) {
			filterLines.set(i, new FilterLine("Filtre n " + (i + 1) + " :", columnNames));
			box.add((FilterLine) filterLines.get(i));
		}
		this.nbFilterLines = nbLines;
		validate();
	}
}