package venndiagrams;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.NumberFormat;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Vector;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.SwingConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

import windows.MainWindow;
import filters.FilterParam;
import filters.TableParamsFilter;

/**
 * An extension of the potatoes in the form of a table
 * 
 * @author Thomas Hugel, Etienne Fort
 */
public class VennNDiagramPanel extends VennPanel {	
	/**
	 * A line in the VennNDiagramPanel.
	 * Associated to an area in the graphic Venn diagram.
	 */
	protected class ResultsLine extends JPanel {
		protected TableModel filteredTable;
		protected FilterParam[] lineParams;
		private JLabel nbLabel;
		private JLabel ratioLabel;
		private NumberFormat ratioNumberFormat;
		public ResultsLine(FilterParam[] paramsArray) {
			lineParams = paramsArray;
			Box titles = Box.createVerticalBox();
			filteredTable = originalTable;
			for (int i = 0; i < lineParams.length; i++) {
				final JLabel label = new JLabel(lineParams[i].getColumnText());
				titles.add(label);
				final int num = i;
				lineParams[i].addChangeListener(new ChangeListener(){
					public void stateChanged(ChangeEvent arg0) {
						label.setText(lineParams[num].getColumnText());
					}
					
				});
			}
			filteredTable = new TableParamsFilter(filteredTable, lineParams);
			add(titles);
			
			add(nbLabel = new JLabel(Integer.toString(filteredTable.getRowCount())));
			ratioNumberFormat = NumberFormat.getPercentInstance();
			ratioNumberFormat.setMaximumFractionDigits(2);
			add(ratioLabel = new JLabel(getRatioString()));
			final JButton seeButton = new JButton("Voir");
			add(seeButton);
			seeButton.setBackground(getRatioColor());
			seeButton.addActionListener(new ActionListener() {
				/* (non-Javadoc)
				 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
				 */
				public void actionPerformed(ActionEvent e) {
					MainWindow.showNewTableWindow("", filteredTable, lineParams);
				}
			});
			filteredTable.addTableModelListener(new TableModelListener() {
				/* (non-Javadoc)
				 * @see javax.swing.event.TableModelListener#tableChanged(javax.swing.event.TableModelEvent)
				 */
				public void tableChanged(TableModelEvent arg0) {
					if (filteredTable != null) {						
						ratioLabel.setText(getRatioString());
						nbLabel.setText(Integer.toString(filteredTable.getRowCount()));
						seeButton.setBackground(getRatioColor());
			
					}
				}
			});
		}
		/**
		 * @return the formatted String of the ratio.
		 */
		public String getRatioString() {
			return ratioNumberFormat.format((double) filteredTable.getRowCount()
					/ (double) originalTable.getRowCount());
		}
		/**
		 * result < 5% : color red;
		 * result > 95% : color cyan;
		 *
		 * @return the color corresponding to the filtering result
		 */
		public Color getRatioColor() {
			double d =
				(double) filteredTable.getRowCount()
					/ (double) originalTable.getRowCount();
			if (d < .05)
				return Color.RED;
			else if (d > .95)
				return Color.CYAN;
			else
				return Color.WHITE;
		}
	}
	/**
	 * Constructs a panel in which the subsets of the columns set
	 * are represented line by line.
	 * 
	 * @param table
	 *            the filteredTable from which the diagram is constructed.
	 * @param inputParameters
	 *            the paramaters of the diagram.
	 */
	public VennNDiagramPanel(TableModel table, FilterParam[] inputParameters) {
		super(table, inputParameters);
		Vector[] subsets = subSets(inputParameters);		
		Arrays.sort(subsets, new Comparator(){
			/* (non-Javadoc)
			 * @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
			 */
			public int compare(Object arg0, Object arg1) {				
				return (((Vector)arg1).size()-((Vector)arg0).size());
			}
		});
		Box box;
		add(box = Box.createVerticalBox());
		int oldNbParams = 0;
		for (int i = 0; i < subsets.length; i++) {
			FilterParam[] newParams = 
				(FilterParam[]) subsets[i].toArray(new FilterParam[subsets[i].size()]);
			if ((newParams.length != oldNbParams) && (oldNbParams != 0))
				box.add(new JSeparator(SwingConstants.HORIZONTAL));
			oldNbParams = newParams.length;
			box.add(new ResultsLine(newParams));
		}
	}
	
}
