package venndiagrams;
import java.awt.Color;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.JPanel;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

import filters.FilterParam;

/**
 * Abstract class for VennDiagrams. It provides some useful methods!
 * 
 * @author Etienne Fort
 */
public abstract class VennPanel extends JPanel {
	protected FilterParam[] filterParams;
	protected TableModel originalTable;
	/**
	 * Sets some proper parameters
	 * 
	 * @param table
	 * 				the support of the calculation
	 * @param params
	 * 				the input parameters
	 */
	public VennPanel(TableModel table, FilterParam[] params) {
		filterParams = params;
		originalTable = table;
		originalTable.addTableModelListener(new TableModelListener(){
			public void tableChanged(TableModelEvent arg0) {
				repaint();		
			}	
		});
	}
	/**
	 * @param d
	 *            the percentage (between 0 and 1).
	 * @return the appropriate color to display according to a percentage of
	 * filtered elements.
	 */
	public Color getThresholdColor(double d){
		if (d < .05) return Color.RED;
		else if (d > .95) return Color.BLUE;
		else return Color.BLACK;
	}
	/**
	 * Calculates all subsets of a finite set. In a word, result is P(E).
	 * 
	 * @param elements
	 *            the set E
	 * @return an array with all subsets of E.
	 */
	public Vector[] subSets(Object[] elements) {
		Vector result = new Vector();
		result.add(new Vector());
		for (int i = 0; i < elements.length; i++) {
			Vector result2 = (Vector) result.clone();
			for (Enumeration e = result.elements(); e.hasMoreElements();) {
				Vector ensemble = (Vector) (((Vector) e.nextElement()).clone());
				ensemble.add(elements[i]);
				result2.add(ensemble);
			}
			result = result2;
		}
		result.remove(0); //to ignore empty set
		return (Vector[]) result.toArray(new Vector[result.size()]);
	}
}
