package windows;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JSplitPane;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.table.TableModel;

import venndiagrams.FiltersParamsPanel;
import venndiagrams.Venn3DiagramPanel;
import venndiagrams.VennNDiagramPanel;

import filters.FilterParam;

/**
 * Window for the diagrams maintaining an area for input parameters on the
 * left while updating dynamically the diagram area on the right.
 * 
 * @author Etienne Fort
 */
public class DiagramWindow extends JFrame {
	private static final int MAX_FILTERS = 10;
	/**
	 * Creates a new DiagramWindow and shows it on the right side.
	 * 
	 * @param owner
	 *            the frame from which this frame is displayed.
	 * @param title
	 *            the title of this window.
	 * @param table
	 *            the table from which the filters will operate.
	 */
	public static void showDiagramWindow(JFrame owner, String title,
			TableModel table) {
		//Creates and sets up the window.
		JFrame frame = new DiagramWindow(table);
		//Displays the window.
		frame.setTitle(title);
		frame.pack();
		frame.setLocationRelativeTo(owner);
		frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		frame.setVisible(true);
	}
	private JSpinner nbFiltersSpin;
	private TableModel originalTable;
	/**
	 * Panel containing the <code>FilterLine</code> lines of parameters. 
	 */
	protected FiltersParamsPanel parametersPanel;
	private JScrollPane scrollPane;
	private JScrollPane viewScrollpane;
	private JSplitPane splitPane;
	/**
	 * Creates a new DiagramWindow on the TableModel table.
	 * 
	 * @param table
	 *            the table from which the filters will operate.
	 */
	public DiagramWindow(TableModel table) {
		originalTable = table;
		parametersPanel = new FiltersParamsPanel(2, originalTable);
		JPanel topPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
		nbFiltersSpin = new JSpinner(new SpinnerNumberModel(2, 1, MAX_FILTERS, 1));
		nbFiltersSpin.addChangeListener(new ChangeListener() {
			/* (non-Javadoc)
			 * @see javax.swing.event.ChangeListener#stateChanged(javax.swing.event.ChangeEvent)
			 */
			public void stateChanged(ChangeEvent arg0) {
				reallocateView();
			}
		});
		topPanel.add(new JLabel("Nombre de filtres : "));
		topPanel.add(nbFiltersSpin);
		final JCheckBox act = new JCheckBox("Actualiser");
		act.setSelected(true);
		act.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				FilterParam.ALERT_LISTENERS = act.isSelected();
				FilterParam[] parameters = parametersPanel.getParameters();
				for (int i = 0; i < parameters.length; i++) {
					parameters[i].fireStateChanged();
				}
			}
		});
		FilterParam.ALERT_LISTENERS = act.isSelected();
		topPanel.add(act);
		JPanel inputpanel = new JPanel(new BorderLayout());
		inputpanel.add(scrollPane = new JScrollPane(parametersPanel));
		inputpanel.add(topPanel, BorderLayout.NORTH);
		getContentPane().add(
				splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
						inputpanel, viewScrollpane = new JScrollPane()));
		reallocateView();
	}
	/**
	 * Reallocates the proper view conreponding to the number of filters chosen
	 * in the Spin control swing component
	 */
	public void reallocateView() {
		int n = ((Integer) nbFiltersSpin.getValue()).intValue();
		parametersPanel.setNbFilterLines(n);
		JPanel viewPanel;
		if (n <= 3)
			viewPanel = new Venn3DiagramPanel(originalTable, parametersPanel.getParameters());
		else
			viewPanel = new VennNDiagramPanel(originalTable, parametersPanel.getParameters());
		viewScrollpane.setViewportView(viewPanel);
		scrollPane.validate();
		viewScrollpane.validate();
		splitPane.validate();
		repaint();
	}
}
