package windows;
import filters.FilterParam;
import graphs.GraphPanel;
import graphs.GraphPanel1;
import graphs.GraphPanel2;
import graphs.GraphsParamsPanel;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JSplitPane;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;
/**
 * Window for the graphs maintaining an area for input parameters on the
 * left while updating dynamically the graph area on the right. 
 * 
 * @see DiagramWindow
 * 
 * @author Julien Robert, Thomas Hugel
 */
public class GraphWindow extends JFrame {
	/**
	 * Creates a new GraphWindow and shows it at the proper place.
	 * 
	 * @param owner
	 *            the frame from which this frame is displayed.
	 * @param title
	 *            the title of this window.
	 * @param table
	 *            the table from which the filters will operate.
	 */
	public static void showGraphWindow(JFrame owner, TableModel table) {
		//Creates and sets up the window.
		final JFrame frame = new GraphWindow(table,owner);
		//Displays the window.
		frame.pack();
		frame.setTitle("Graphiques : " + owner.getTitle());
		frame.setLocationRelativeTo(owner);
		frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		frame.setVisible(true);
	}
	private JPanel squareAlertOutOfDate;
	private JButton buttonDraw, buttonPrint;
	/**
	 * Panel containing the <code>FilterLine</code> lines of parameters.
	 */
	protected GraphsParamsPanel myGraphicsPanel;
	private int nbGraphs;
	private TableModel originalTable;
	private int precision;
	private JSpinner precisionSpinner;
	private JScrollPane viewScrollpane; 
	private JScrollPane scrollPane;
	private JSplitPane splitPane;
	private GraphPanel viewPanel;
	private JFrame owner;
	/**
	 * Creates a new DiagramWindow on the TableModel table.
	 * 
	 * @param table
	 *            the table from which the filters will operate.
	 */
	public GraphWindow(TableModel table,JFrame father){
		originalTable = table;
		owner= father;
		originalTable.addTableModelListener(new TableModelListener(){
			public void tableChanged(TableModelEvent arg0) {
				setTitle("Graphiques : " + owner.getTitle());
				setGraphPanelUpToDate(false);	
			}		
		});
		precision = 100;
		myGraphicsPanel = new GraphsParamsPanel(nbGraphs, originalTable);
		myGraphicsPanel.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent evt) {
				setGraphPanelUpToDate(false);
			}
		});
		JPanel nbpanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
		precisionSpinner = new JSpinner(new SpinnerNumberModel(precision, 10, 500, 10));
		buttonDraw = new JButton("Afficher");
		precisionSpinner.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent arg0) {
				precision = ((Integer) precisionSpinner.getValue()).intValue();
				setGraphPanelUpToDate(false);
			}
		});
		
		buttonDraw.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent arg0) {
				reallocateView();
			}
		});
		buttonPrint = new JButton("Imprimer");
		buttonPrint.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				PrinterJob pJob = PrinterJob.getPrinterJob();
				if (pJob.printDialog()) {
					pJob.setPrintable(viewPanel);
					try {
						pJob.print();
					} catch (PrinterException e) {
						System.out.print("Impression impossible");
						JOptionPane.showMessageDialog(null,
								"Impression impossible",
								"L'impression a chou",
								JOptionPane.ERROR_MESSAGE);
						e.printStackTrace();
					}
				}
			}
		});
		
		nbpanel.add(precisionSpinner);
		nbpanel.add(buttonDraw);
		nbpanel.add(buttonPrint);
		nbpanel.add(squareAlertOutOfDate = new JPanel());
		setGraphPanelUpToDate(true);
		JPanel inputpanel = new JPanel(new BorderLayout());
		inputpanel.add(scrollPane = new JScrollPane(myGraphicsPanel));
		inputpanel.add(nbpanel, BorderLayout.NORTH);
		getContentPane().add(
				splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
						inputpanel, viewScrollpane = new JScrollPane()));
		JPanel panel = new JPanel();
		panel.setPreferredSize(GraphPanel.GRAPH_DIMENSION);
		viewScrollpane.setViewportView(panel);
	}
	/**
	 * Sets the proper color to the alert signal
	 */
	private void setGraphPanelUpToDate(boolean b) {
		squareAlertOutOfDate.setBackground(b ? Color.GREEN : Color.RED);
		squareAlertOutOfDate.repaint();
	}
	
	
	/**
	 * Reaction to the events that modify the parameters.
	 * Brings the graphs up to date.
	 */
	public void reallocateView() {
		//JPanel viewPanel;
		nbGraphs = myGraphicsPanel.getNbCurves();
		FilterParam[] params = myGraphicsPanel.getParameters();
		if (nbGraphs == 1) {
			int num = myGraphicsPanel.getNumCurve();
			viewPanel = new GraphPanel1(originalTable,
					params[num], precision,
					myGraphicsPanel.getThresholdRange1(),
					(num == 0) ? Color.RED : Color.BLUE);
		} else {				//nbGraphs == 0 or 2
			viewPanel = new GraphPanel2(params,
					originalTable, precision, myGraphicsPanel.getThresholdRange1(), 
					myGraphicsPanel.getThresholdRange2());
		}
		viewScrollpane.setViewportView(viewPanel);
		viewScrollpane.validate();
		
		//pack();
		repaint();
		setGraphPanelUpToDate(true);
	}
}